% -----------------------------------------------

%
% Installation instructions:
% 
% 1. Copy the code that follows this comment and add it to the 
%    end of your .jedrc file.
%
% 2. Un-comment and change the key bindings at the end of the
%    code to keys that are convenient for you.
%
% 3. To enable the text_mode_wrap_hook() and txt_adorn_title() for Text mode,
%    add something like the following to your global_mode_hook() or to
%    your text_mode_hook():
%
%       define global_mode_hook (hook_name)
%       {
%           o
%           o
%           o
%           else if (hook_name == "text_mode_hook")
%           {
%               local_setkey("txt_adorn_title", "\ezt");
%               set_buffer_hook ("wrap_hook", &text_mode_wrap_hook);
%           }
%           o
%           o
%           o
%       }
%

% -----------------------------------------------

%
% Macros for reStructuredText and Docutils.
% 
% Line wrapping for lines that start a bullet list or enumerated list.
% See doc/txt/hooks.txt.
% This code originated from hooks.txt.
% 
static define text_mode_wrap_hook ()
{
    variable p;
    push_spot ();
    go_up(1);
    bol ();
    skip_white ();
    p = what_column ();
    if ((looking_at("- ")) or 
        (looking_at("* ")) or
        (looking_at("+ "))
        )
    {
        go_down(1);
        bol ();
        skip_white ();
        p = what_column ();
        bol ();
        trim ();
        whitespace (p + 1);
    }
    else
        if (
            (looking_at("1. ")) or
            (looking_at("2. ")) or
            (looking_at("3. ")) or
            (looking_at("4. ")) or
            (looking_at("5. ")) or
            (looking_at("6. ")) or
            (looking_at("7. ")) or
            (looking_at("8. ")) or
            (looking_at("9. "))
            )
        {
        go_down(1);
        bol ();
        skip_white ();
        p = what_column ();
        bol ();
        trim ();
        whitespace (p + 2);
        }
    pop_spot ();
}

%
% Underline (adorn) a title (Docutils style).
% With any prefix argument, also add adornment *above* the title.
%
define txt_adorn_title()
{
    variable loc1;
    variable loc2;
    variable idx;
    variable char_count;
    variable adornment_char;
    variable arg;
    arg = prefix_argument(-1);
    adornment_char = read_mini("Enter adornment character:", "=", "");
    bol();
    loc1 = what_column();
    eol();
    loc2 = what_column();
    char_count = loc2 - loc1;
    
    if (arg != -1)
    {
        bol();
        for (idx = 0; idx < char_count; idx++)
        {
            insert(adornment_char);
        }
        insert("\n");
        eol();
    }
    insert("\n");
    for (idx = 0; idx < char_count; idx++)
    {
        insert(adornment_char);
    }
    insert("\n\n");
}

% -----------------------------------------------

%
% Switch to previous or next buffer.
%   This code originated in ide_next_buffer() in ide.sl.
%   
define next_buffer (previous)
{
    variable n, buf;
    n = buffer_list (); % get the buffers on the stack
    if (previous)
    {
        _stk_reverse (n-1);
    }
    loop (n) {
        buf = ();
        n--;
        % Skip *scratch* and other buffers that are not of interest.
        if ((buf[0] == '*') or (buf[0] == ' '))
        {
            continue;
        }
        sw2buf (buf);
        _pop_n (n);
        return;
    }
}

% -----------------------------------------------

autoload("get_word", "txtutils");

define search_current_word(direction)
{
    variable word;
    variable mark;
    variable result;
    variable s1;
    mark = create_user_mark();
    skip_word();
    bskip_word();
    word = get_word();
    if (direction == 1)
    {
        while (1)
        {
            go_right_1();
            result = fsearch(word);
            if (result == 0)
            {
                break;
            }
            % Search again if the word is a substring of a longer word.
            s1 = get_word();
            !if (strcmp(s1, word))
            {
                break;
            }
        }
    }
    else
    {
        while (1)
        {
            result = bsearch(word);
            if (result == 0)
            {
                break;
            }
            % Search again if the word is a substring of a longer word.
            s1 = get_word();
            !if (strcmp(s1, word))
            {
                break;
            }
        }
    }
    if (result == 0)
    {
        goto_user_mark(mark);
        message(word + " not found");
    }
}

% -----------------------------------------------

autoload("yp_copy_region_as_kill", "yankpop");
autoload("mark_word", "txtutils");

define copy_word_as_kill()
{
    variable s1;
    push_spot();
    mark_word();
    () = dupmark();
    s1 = bufsubstr();
    yp_copy_region_as_kill();
    pop_spot();
    message("\"" + s1 + "\" copied to kill ring");
}

define copy_line_as_kill()
{
    variable s1;
    push_spot();
    bol();
    set_mark_cmd();
    down(1);
    yp_copy_region_as_kill();
    pop_spot();
    message("line copied to kill ring");
}

define duplicate_line()
{
    variable col;
    push_spot();
    col = what_column();
    bol();
    push_mark();
    down_1();
    yp_copy_region_as_kill();
    yp_yank();
    pop_spot();
    down_1();
    goto_column(col);
}

% -----------------------------------------------

%
% Bind Alt-. and Alt-, (Meta-period and Meta-comma) to next_buffer().
% setkey ("next_buffer (0)", "\e,");
% setkey ("next_buffer (1)", "\e.");

%
% Bind to Alt-z+Alt-comma (alt-z then alt-comma) and Alt-z+Alt-period.
% setkey ("search_current_word(-1)", "\ez\e,");
% setkey ("search_current_word(1)", "\ez\e.");

% setkey ("copy_word_as_kill()", "\eze");
% setkey ("copy_line_as_kill()", "\ezr");
% setkey("duplicate_line", "\ezl");

% -----------------------------------------------

%
% Load the file whose name is under the cursor into a buffer.
%
define load_file_under_cursor()
{
   variable s1;
   variable wordchars1;
   push_spot();
   wordchars1 = "-_A-Za-z0-9./";
   mark_word(wordchars1);
   %() = dupmark();
   s1 = bufsubstr();
   %yp_copy_region_as_kill();
   pop_spot();
   find_file(s1);
   message("\"" + s1 + "\" opened");
}

setkey("load_file_under_cursor", "\ezf");

%
% Load the file whose name is under the cursor into a buffer, *and*
%   copy the file name into the copy/kill ring.
%
define load_file_under_cursor()
{
   variable s1;
   variable wordchars1;
   push_spot();
   wordchars1 = "-_A-Za-z0-9./";
   mark_word(wordchars1);
   () = dupmark();
   s1 = bufsubstr();
   yp_copy_region_as_kill();
   pop_spot();
   find_file(s1);
   message("\"" + s1 + "\" opened");
}

setkey("load_file_under_cursor", "\ezf");


% -----------------------------------------------

